

proc Set_VF48_Options {} {
    global Options

    load_Options

#   if {![info exists Options(vf48.channels)]} {set Options(vf48.channels) 32768}
#   if {![info exists Options(vf48.offset)]} {set Options(vf48.offset) 0}
#   if {![info exists Options(vf48.shift)]} {set Options(vf48.shift) 1}

    update_Options
}

proc Dump_Raw_VME_Data_VF48 {} {

   set info "VF48\n"

   foreach s {0 1 2 3 4} {append info [next_section $s]}

   append info "\n"

   return $info
}

proc next_section {n} {

     set info ""
     set s [expr (($n * 100) << 2) + 1]

     execute-command 19 "Getting Raw Data for VF48" "7 $s" 2

     foreach word [read-register NetVar.EXEC.Response] {
         set word 0x$word
         if {$word == 0xaa5555aa} then {
             append info "$word - end of section\n"
             break
         }
         if {$word == 0x55aaaa55} then {
             append info "$word - end of fragment\n"
             continue
         }
         set type [expr $word >> 28 & 15]
         switch $type {
           0 {
             set event_type "data"
             set s0 [expr $word & 0x3ff]
             set s1 [expr $word >> 16 & 0x3ff]
             append info "[format "0x%8x    %8s   sample n+1 = 0x%3x;  sample n = 0x%3x" $word $event_type $s1 $s0]"
           }
           4 {
             set event_type "CFD Time"
             set vl [expr $word & 0x00ffffff]
             append info "[format "0x%8x    %8s   0x%6x" $word $event_type $vl]"
           }
           5 {
             set event_type "Charge"
             set vl [expr $word & 0x00ffffff]
             append info "[format "0x%8x    %8s   0x%6x" $word $event_type $vl]"
           }
           8 {
             set event_type "Header"
             set trig [expr $word & 0x00ffffff]
             append info "[format "0x%8x    %8s;   trigger number= 0x%6x" $word $event_type $trig]"
           }
           10 {
             set event_type "Time Stamp"
             set vl [expr $word & 0x00ffffff]
             append info "[format "0x%8x    %8s   0x%6x" $word $event_type $vl]"
           }
           12 {
             set event_type "Channel ID"
             set group [expr $word >> 4 & 7]
             set channel [expr $word & 15]
             append info "[format "0x%8x    %8s   group= %d;   channel= %d" $word $event_type $group $channel]"
           }
           14 {
             set event_type "Trailer"
             set trig [expr $word & 0x00ffffff]
             append info "[format "0x%8x    %8s;   trigger number= 0x%6x" $word $event_type $trig]"
           }
           default {append info "[format "0x%8x" $word]"}
         }
         append info "\n"
     }

   return $info
}

proc define_VF48_config {} {

   set maxmodule [inform vmecontrol inquire vf48count]
   if {[string length $maxmodule] > 2} {set maxmodule 0}

   for {set i 1} {$i <= $maxmodule} {incr i} {

      set VMEserver [inform vmecontrol VMEmodule2server vf48#$i]
      set z [catch {eg "(register :name '/$VMEserver/vf48#$i.CSR'  ?object)" } m]
      if {$z!=0} then {
          display-eg-error-report self $m "Attempting to locate register vf48#$i.CSR"
          clear-footer
          error $m
      }
      DefineConfig "48 $i [string range [first $m] 2 end]"
   }
}


set VF48Group1Base 128
set VF48Group2Base 160

proc create_VF48_spectra {} {
   global SpectrumServer
   global VF48Group1Base VF48Group2Base

#   for the VF48 48 channel module

   set maxmodule [inform vmecontrol inquire vf48count]
   if {[string length $maxmodule] > 2} {set maxmodule 0}
   set maxadc 48

   for {set i 1} {$i <= $maxmodule} {incr i} {

#    for the timestamp

         set z [catch {eg "(spectrum :path '/$SpectrumServer' :range 65536 :name 'vf48#$i.timestamp.h' !new)" } m]
         if {$z!=0 && $m==0x30004} then {set z 0}
         if {$z!=0} then {
            display-eg-error-report self $m "Attempting to create spectrum vf48#$i.timestamp.h"
            clear-footer
            error $m
         } else {
            map_spectrum  "vf48#$i.timestamp.h [expr $VF48Group1Base + ($i - 1)] 0 0 0"    ;# Timestamp H - item 0
         }

         set z [catch {eg "(spectrum :path '/$SpectrumServer' :range 65536 :name 'vf48#$i.timestamp.m' !new)" } m]
         if {$z!=0 && $m==0x30004} then {set z 0}
         if {$z!=0} then {
            display-eg-error-report self $m "Attempting to create spectrum vf48#$i.timestamp.m"
            clear-footer
            error $m
         } else {
            map_spectrum  "vf48#$i.timestamp.m [expr $VF48Group1Base + ($i - 1)] 1 0 0"    ;# Timestamp M - item 1
         }

         set z [catch {eg "(spectrum :path '/$SpectrumServer' :range 65536 :name 'vf48#$i.timestamp.l' !new)" } m]
         if {$z!=0 && $m==0x30004} then {set z 0}
         if {$z!=0} then {
            display-eg-error-report self $m "Attempting to create spectrum vf48#$i.timestamp.l"
            clear-footer
            error $m
         } else {
            map_spectrum  "vf48#$i.timestamp.l [expr $VF48Group1Base + ($i - 1)] 2 0 0"    ;# Timestamp L - item 2
         }
    
      for {set j 0} {$j < $maxadc} {incr j} {

#    for the raw data trace 

         set vl 256     ;# use default
         set z [catch {eg "(spectrum :path '/$SpectrumServer' :range $vl :name 'vf48#$i.ch$j' !new)" } m]

#    Allow "Spectrum already exists" so that further spectra can be created
         if {$z!=0 && $m==0x30004} then {set z 0}
         if {$z!=0} then {
            display-eg-error-report self $m "Attempting to create spectrum vf48#$i.ch$j"
            clear-footer
            error $m
         } else {
            map_spectrum  "vf48#$i.ch$j [expr 256 + $j + ($i * 48)] 0 0 0"
         }

#    for the CFD time

         set z [catch {eg "(spectrum :path '/$SpectrumServer' :range 256 :name 'vf48#$i.ch$j.cfd.h' !new)" } m]
         if {$z!=0 && $m==0x30004} then {set z 0}
         if {$z!=0} then {
            display-eg-error-report self $m "Attempting to create spectrum vf48#$i.ch$j.cfd.h"
            clear-footer
            error $m
         } else {
            map_spectrum  "vf48#$i.ch$j.cfd.h [expr $VF48Group2Base + ($i - 1)*48 + $j] 0 0 0"    ;# CFD H - item 0
         }

         set z [catch {eg "(spectrum :path '/$SpectrumServer' :range 65536 :name 'vf48#$i.ch$j.cfd.l' !new)" } m]
         if {$z!=0 && $m==0x30004} then {set z 0}
         if {$z!=0} then {
            display-eg-error-report self $m "Attempting to create spectrum vf48#$i.ch$j.cfd.l"
            clear-footer
            error $m
         } else {
            map_spectrum  "vf48#$i.ch$j.cfd.l [expr $VF48Group2Base + ($i - 1)*48 + $j] 1 0 0"    ;# CFD L - item 1
         }

#    for the charge

         set z [catch {eg "(spectrum :path '/$SpectrumServer' :range 256 :name 'vf48#$i.ch$j.q.h' !new)" } m]
         if {$z!=0 && $m==0x30004} then {set z 0}
         if {$z!=0} then {
            display-eg-error-report self $m "Attempting to create spectrum vf48#$i.ch$j.q.h"
            clear-footer
            error $m
         } else {
            map_spectrum  "vf48#$i.ch$j.q.h [expr $VF48Group2Base + ($i - 1)*48 + $j] 2 0 0"    ;# Q H - item 2
         }

         set z [catch {eg "(spectrum :path '/$SpectrumServer' :range 65536 :name 'vf48#$i.ch$j.q.l' !new)" } m]
         if {$z!=0 && $m==0x30004} then {set z 0}
         if {$z!=0} then {
            display-eg-error-report self $m "Attempting to create spectrum vf48#$i.ch$j.q.l"
            clear-footer
            error $m
         } else {
            map_spectrum  "vf48#$i.ch$j.q.l [expr $VF48Group2Base + ($i - 1)*48 + $j] 3 0 0"    ;# Q L - item 3
         }

      }   ;# of loop on j
   }
}
